package com.jiuqi.sql.impl;

import com.jiuqi.util.JqLib;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Calendar;
import java.util.Date;

public class RW {
   private static final int dayOfYear = 365;
   private static final int dayOf4Year = 1461;
   private static final int dayOfCentury = 36524;
   private static final int dayOf4Century = 146097;
   private static final int[][] monthDays = new int[][]{{31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}, {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}};
   private static final int hoursPerDay = 24;
   private static final int minsPerHour = 60;
   private static final int secsPerMin = 60;
   private static final int mSecsPerSec = 1000;
   private static final int minsPerDay = 1440;
   private static final int secsPerDay = 86400;
   private static final int mSecsPerDay = 86400000;

   private static int[] decodeDelphiDateFully(double dateTime) {
      int[] result = new int[5];
      int t = (int)dateTime + 693594;
      if (t > 0) {
         result[3] = t % 7 + 1;
         --t;

         int y;
         for(y = 1; t >= 146097; y += 400) {
            t -= 146097;
         }

         int i = t / '躬';
         int d = t % '躬';
         if (i == 4) {
            --i;
            d += 36524;
         }

         y += i * 100;
         i = d / 1461;
         d %= 1461;
         y += i * 4;
         i = d / 365;
         d %= 365;
         if (i == 4) {
            --i;
            d += 365;
         }

         y += i;
         result[4] = y % 4 != 0 || y % 100 == 0 && y % 400 != 0 ? 0 : 1;
         int[] dayTable = monthDays[result[4]];
         int m = 1;

         while(true) {
            i = dayTable[m - 1];
            if (d < i) {
               result[0] = y;
               result[1] = m - 1;
               result[2] = d + 1;
               break;
            }

            d -= i;
            ++m;
         }
      }

      return result;
   }

   private static double encodeDelphiDateFully(int year, int dayOfYear) {
      int dayCount = (year - 1 - 1899) * 365;
      int leapYearCount = 0;

      for(int i = 1900; i <= year - 1; ++i) {
         if (JqLib.isLeapYear(i)) {
            ++leapYearCount;
         }
      }

      dayCount += leapYearCount;
      ++dayCount;
      return (double)(dayCount + dayOfYear);
   }

   private static int[] decodeDelphiTime(double dateTime) {
      int[] result = new int[4];
      int time = (int)(8.64E7 * (dateTime - (double)((int)dateTime)));
      int minCount = time / '\uea60';
      int mSecCount = time % '\uea60';
      result[0] = minCount / 60;
      result[1] = minCount % 60;
      result[2] = mSecCount / 1000;
      result[3] = mSecCount % 1000;
      return result;
   }

   private static double encodeDelphiTime(int hour, int minute, int second, int msec) {
      int minCount = 60 * hour + minute;
      int mSecCount = 1000 * second + msec;
      int time = '\uea60' * minCount + mSecCount;
      return (double)time / 8.64E7;
   }

   private static Date decodeDelphiDateTime(double dateTime) {
      int[] date = decodeDelphiDateFully(dateTime);
      int[] time = decodeDelphiTime(dateTime);
      Calendar calendar = Calendar.getInstance();
      calendar.setTimeInMillis(0L);
      calendar.set(date[0], date[1], date[2], time[0], time[1], time[2]);
      return calendar.getTime();
   }

   private static double encodeDelphiDateTime(Date date) {
      Calendar calendar = Calendar.getInstance();
      calendar.setTimeInMillis(0L);
      calendar.setTime(date);
      int year = calendar.get(1);
      int dayOfYear = calendar.get(6);
      int hour = calendar.get(10);
      int minute = calendar.get(12);
      int sec = calendar.get(13);
      int mSec = calendar.get(14);
      return encodeDelphiDateFully(year, dayOfYear) + encodeDelphiTime(hour, minute, sec, mSec);
   }

   public static byte read(InputStream stream) throws IOException {
      return (byte)stream.read();
   }

   public static void write(OutputStream stream, byte value) throws IOException {
      stream.write(value);
   }

   public static boolean readBoolean(InputStream stream) throws IOException {
      return stream.read() > 0;
   }

   public static void writeBoolean(OutputStream stream, boolean value) throws IOException {
      stream.write(value ? 1 : 0);
   }

   public static boolean readBoolean(byte[] data, int index) {
      return data[index] == 84 || data[index] == 116 || data[index] == 49;
   }

   public static void writeBoolean(byte[] data, int index, boolean value) {
      if (value) {
         data[index] = 84;
      } else {
         data[index] = 70;
      }

   }

   public static String readString(InputStream stream, int len) throws IOException {
      byte[] str = new byte[len];
      stream.read(str, 0, len);
      return bytesToString(str);
   }

   public static void writeString(OutputStream stream, String value, int len) throws IOException {
      int l = 0;
      if (value != null) {
         byte[] str = value.getBytes();
         stream.write(str, 0, str.length > len ? len : str.length);
         if (str.length > len) {
            return;
         }

         l = str.length;
      }

      for(int i = 1; i <= len - l; ++i) {
         stream.write(0);
      }

   }

   private static String bytesToString(byte[] data) {
      if (data != null && data.length != 0) {
         for(int i = data.length - 1; i >= 0; --i) {
            if (data[i] != 0 && data[i] != 32) {
               return new String(data, 0, i + 1);
            }
         }

         return "";
      } else {
         return null;
      }
   }

   public static String readString(byte[] data, int index, int len) {
      byte[] str = new byte[len];
      System.arraycopy(data, index, str, 0, len);
      return bytesToString(str);
   }

   public static void writeString(byte[] data, int index, String value, int len) {
      byte[] dataToWrite = value.getBytes();
      System.arraycopy(dataToWrite, 0, data, index, dataToWrite.length > len ? len : dataToWrite.length);
      if (len > dataToWrite.length) {
         byte[] restData = new byte[len - dataToWrite.length];
         int ind = dataToWrite.length + index;
         System.arraycopy(restData, 0, data, ind, restData.length);
      }

   }

   public static Date readDateTime(byte[] data, int index) {
      double datetime = readDouble(data, index);
      return decodeDelphiDateTime(datetime);
   }

   public static Date readDateTime(InputStream stream) throws IOException {
      double datetime = readDouble(stream);
      return decodeDelphiDateTime(datetime);
   }

   public static void writeDateTime(byte[] data, int index, Date dateValue) {
      double datetime = encodeDelphiDateTime(dateValue);
      writeDouble(data, index, datetime);
   }

   public static void writeDateTime(OutputStream stream, Date dateValue) throws IOException {
      double datetime = encodeDelphiDateTime(dateValue);
      writeDouble(stream, datetime);
   }

   public static void writeDateTime(OutputStream stream, long dateValue) throws IOException {
      writeLong(stream, dateValue);
   }

   public static void writeDateTime(byte[] data, int index, long dateValue) {
      writeLong(data, index, dateValue);
   }

   public static double readDouble(InputStream stream) throws IOException {
      return Double.longBitsToDouble(readLong(stream));
   }

   public static void writeDouble(OutputStream stream, double value) throws IOException {
      writeLong(stream, Double.doubleToLongBits(value));
   }

   public static double readDouble(byte[] data, int index) {
      return Double.longBitsToDouble(readLong(data, index));
   }

   public static void writeDouble(byte[] data, int index, double value) {
      writeLong(data, index, Double.doubleToLongBits(value));
   }

   public static float readFloat(InputStream stream) throws IOException {
      return Float.intBitsToFloat(readInt(stream));
   }

   public static void writeFloat(OutputStream stream, float value) throws IOException {
      writeInt(stream, Float.floatToIntBits(value));
   }

   public static float readFloat(byte[] data, int index) {
      return Float.intBitsToFloat(readInt(data, index));
   }

   public static void writeFloat(byte[] data, int index, float value) {
      writeInt(data, index, Float.floatToIntBits(value));
   }

   public static int readInt(InputStream stream) throws IOException {
      int b1 = stream.read() & 255;
      int b2 = stream.read() & 255;
      int b3 = stream.read() & 255;
      int b4 = stream.read() & 255;
      int result = (b4 << 0) + (b3 << 8) + (b2 << 16) + (b1 << 24);
      return result ^ Integer.MIN_VALUE;
   }

   public static void writeInt(OutputStream stream, int value) throws IOException {
      value ^= Integer.MIN_VALUE;
      byte[] b = new byte[]{(byte)(value >> 24 & 255), (byte)(value >> 16 & 255), (byte)(value >> 8 & 255), (byte)(value >> 0)};
      stream.write(b);
   }

   public static int readInt(byte[] data, int index) {
      int b1 = data[index] & 255;
      int b2 = data[index + 1] & 255;
      int b3 = data[index + 2] & 255;
      int b4 = data[index + 3] & 255;
      int result = (b4 << 0) + (b3 << 8) + (b2 << 16) + (b1 << 24);
      return result ^ Integer.MIN_VALUE;
   }

   public static void writeInt(byte[] data, int index, int value) {
      value ^= Integer.MIN_VALUE;
      data[index + 3] = (byte)(value >> 0);
      data[index + 2] = (byte)(value >> 8 & 255);
      data[index + 1] = (byte)(value >> 16 & 255);
      data[index + 0] = (byte)(value >> 24 & 255);
   }

   public static long readLong(InputStream stream) throws IOException {
      long b1 = (long)(stream.read() & 255);
      long b2 = (long)(stream.read() & 255);
      long b3 = (long)(stream.read() & 255);
      long b4 = (long)(stream.read() & 255);
      long b5 = (long)(stream.read() & 255);
      long b6 = (long)(stream.read() & 255);
      long b7 = (long)(stream.read() & 255);
      long b8 = (long)stream.read();
      return (b8 << 56) + (b7 << 48) + (b6 << 40) + (b5 << 32) + (b4 << 24) + (b3 << 16) + (b2 << 8) + (b1 << 0);
   }

   public static void writeLong(OutputStream stream, long value) throws IOException {
      byte[] b = new byte[]{(byte)((int)(value >> 0)), (byte)((int)(value >> 8 & 255L)), (byte)((int)(value >> 16 & 255L)), (byte)((int)(value >> 24 & 255L)), (byte)((int)(value >> 32 & 255L)), (byte)((int)(value >> 40 & 255L)), (byte)((int)(value >> 48 & 255L)), (byte)((int)(value >> 56 & 255L))};
      stream.write(b);
   }

   public static long readLong(byte[] data, int index) {
      long b1 = (long)(data[index] & 255);
      long b2 = (long)(data[index + 1] & 255);
      long b3 = (long)(data[index + 2] & 255);
      long b4 = (long)(data[index + 3] & 255);
      long b5 = (long)(data[index + 4] & 255);
      long b6 = (long)(data[index + 5] & 255);
      long b7 = (long)(data[index + 6] & 255);
      long b8 = (long)data[index + 7];
      return (b8 << 56) + (b7 << 48) + (b6 << 40) + (b5 << 32) + (b4 << 24) + (b3 << 16) + (b2 << 8) + (b1 << 0);
   }

   public static void writeLong(byte[] data, int index, long value) {
      data[index] = (byte)((int)(value >> 0));
      data[index + 1] = (byte)((int)(value >> 8 & 255L));
      data[index + 2] = (byte)((int)(value >> 16 & 255L));
      data[index + 3] = (byte)((int)(value >> 24 & 255L));
      data[index + 4] = (byte)((int)(value >> 32 & 255L));
      data[index + 5] = (byte)((int)(value >> 40 & 255L));
      data[index + 6] = (byte)((int)(value >> 48 & 255L));
      data[index + 7] = (byte)((int)(value >> 56 & 255L));
   }

   public static int readWord(InputStream stream) throws IOException {
      int b1 = stream.read() & 255;
      int b2 = stream.read() & 255;
      return (b2 << 8) + (b1 << 0);
   }

   public static void writeWord(OutputStream stream, int value) throws IOException {
      byte[] b = new byte[]{(byte)value, (byte)(value >> 8 & 255)};
      stream.write(b);
   }

   public static int readWord(byte[] data, int index) {
      int b1 = data[index] & 255;
      int b2 = data[index + 1] & 255;
      return (b2 << 8) + (b1 << 0);
   }

   public static void writeWord(byte[] data, int index, int value) {
      data[index] = (byte)(value >> 0);
      data[index + 1] = (byte)(value >> 8 & 255);
   }

   public static long readDWord(InputStream stream) throws IOException {
      int b1 = stream.read() & 255;
      int b2 = stream.read() & 255;
      int b3 = stream.read() & 255;
      int b4 = stream.read() & 255;
      return (long)((b4 << 24) + (b3 << 16) + (b2 << 8) + (b1 << 0));
   }

   public static void writeDWord(OutputStream stream, long value) throws IOException {
      byte[] b = new byte[]{(byte)((int)(value >> 0)), (byte)((int)(value >> 8 & 255L)), (byte)((int)(value >> 16 & 255L)), (byte)((int)(value >> 24 & 255L))};
      stream.write(b);
   }

   public static long readDWord(byte[] data, int index) {
      int b1 = data[index] & 255;
      int b2 = data[index + 1] & 255;
      int b3 = data[index + 2] & 255;
      int b4 = data[index + 3] & 255;
      return (long)((b4 << 24) + (b3 << 16) + (b2 << 8) + (b1 << 0));
   }

   public static void writeDWord(byte[] data, int index, long value) {
      data[index] = (byte)((int)(value >> 0));
      data[index + 1] = (byte)((int)(value >> 8 & 255L));
      data[index + 2] = (byte)((int)(value >> 16 & 255L));
      data[index + 3] = (byte)((int)(value >> 24 & 255L));
   }
}
